<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use HCM\Util\DateUtil;
use Folha\Entities\Ocorcalendario;



class CalendarioService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
       
    public function deleteCalendario($dataset) {
        $nrcalendario = $dataset["NRCALENDARIO"];
        $feriado = $this->entityManager->getRepository(Repositories::OCORCALENDARIO)->findBy(array("nrcalendario" => $nrcalendario, "nrorg" => $dataset['NRORG']));
        if (isset($feriado[0])) {
            throw new \Exception(Messages::HCM00000_GER_CALENDARIO_ALREADY_EXIST_FERIADO_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $calendario = $this->entityManager->getRepository(Repositories::CALENDARIO)->findOneBy(array("nrcalendario" => $nrcalendario, "nrorg" => $dataset['NRORG']));
            $this->entityManager->remove($calendario);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    /*Funções para sincronização e replicação do calendário*/
    
    //Função que recebe o retorno de uma API REST com todos os feriados daquele 
    public function syncCalendarioFeriado($row){
        //chamada de uma API qeu retorna todos os feriados do ano mês
        $apiCalendar = file_get_contents('https://api.calendario.com.br/?ano=2021&json=true&token=am9mZWxpcGVnMjNAZ21haWwuY29tJmhhc2g9MjU2ODI5MDUx');
        $json_arr = json_decode($apiCalendar, true);
        $msg = array();
        
        foreach($json_arr as $value){
                $teste = $value;
                if($teste["type_code"] == '1'){
                    if($this->validaDataSyncCalendario($value['date'],$row['NRORG'],$row['NRCALENDARIO'])){
                        $this->insereSyncCalendario($teste,$row);
                        $msg[] = "A data ".$value['date']." foi cadastrada no calendário";
                    }else{
                        $msg[] = "A data ".$value['date']." já está cadastrada no calendário";
                    }    
                }
        }
        $this->entityManager->flush();
        return $msg;
    }
     
    public function insereSyncCalendario($value, $row){
        
        try{
            $repCalendario = Repositories::OCORCALENDARIO;
            $objCalendario = new $repCalendario(); 
    
            $numOcorCalendario = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'OCORCALENDARIO', 1, 12, true);
            $numCalendario = $row['NRCALENDARIO'];
            $NrTpcalendario = 12;
            
            $dsobscalend  = isset($value['name']) ? $value['name'] : $row['DSOBSCALEND'];
            $dtcalendario = isset($value['date']) ? $value['date'] : $row['DTCALENDARIO'];
//var_dump($dsobscalend, $dtcalendario, $numCalendario, $numOcorCalendario, $row);
            $objCalendario->setNrocorcalend($numOcorCalendario);
            $objCalendario->setDsobscalend($dsobscalend);
            $objCalendario->setNrcalendario($numCalendario);
            $objCalendario->setDtcalendario(DateUtil::getDataDeString($dtcalendario, DateUtil::FORMATO_BRASILEIRO, true));
            $objCalendario->setNrtpcalendario($NrTpcalendario);
            $objCalendario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $objCalendario->setDtinclusao(DateUtil::getDataAtual());
            $objCalendario->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $objCalendario->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $objCalendario->setDtultatu(DateUtil::getDataAtual());
            $objCalendario->setCdoperultatu($this->hcmParameters->getCdoperador());
            $objCalendario->setNrorg($this->hcmParameters->getNrorgTrabalho());
    
            $this->entityManager->persist($objCalendario);
            unset($objCalendario);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaDataSyncCalendario ($date,$nrorg,$nrcalendario){
        $date = DateUtil::getDataDeString($date, DateUtil::FORMATO_BRASILEIRO, true);
        $obj = $this->entityManager->getRepository(Repositories::OCORCALENDARIO)->findOneBy(array('nrorg'=>$nrorg,'nrcalendario'=>$nrcalendario,'dtcalendario'=>$date));
        if($obj){
            return false;
        }else{
            return true;
        }
    }
    
    public function replicaCalendario($row) {
        $nrorg        = $row['NRORG'];
        $nrcalendario = $row['NRCALENDARIO'];
        $dscalendario = $row['DSCALENDARIO'];
        $anoAnterior  = $row['anoAnterior'];
        $anoAtual     = $row['anoAtual'];
        
        $feriados = $this->getFeriados($nrorg, $nrcalendario, $anoAnterior);
        
        if(count($feriados) == 0)
            return array("Não existem feriados para o \"$dscalendario\" no ano de $anoAnterior. Nenhuma adição foi feita.");
        
        $msg_add = array();
        $msg_not = array();
        
        foreach($feriados as $feriado) {
            
            $feriado['DTCALENDARIO'] = substr($feriado['DTCALENDARIO'], 0, 6) . $anoAtual;
            $feriado['DSOBSCALEND']  = trim($feriado['DSOBSCALEND']);
            
            $res = $this->entityManager->getRepository(Repositories::OCORCALENDARIO)->findOneBy(
                array('nrorg'        => $nrorg,
                      'nrcalendario' => $nrcalendario,
                      'dtcalendario' => DateUtil::getDataDeString($feriado['DTCALENDARIO'], DateUtil::FORMATO_BRASILEIRO, true)));
            
            $msg = $feriado['DTCALENDARIO'].": " . ( strlen($feriado['DSOBSCALEND']) > 21 ? substr($feriado['DSOBSCALEND'], 0, 20)."." : $feriado['DSOBSCALEND'] );
            
            if(isset($res)) {
                $msg_not[] = $msg;
            } else {
                $this->insereSyncCalendario(array(), $feriado);
                $msg_add[] = $msg;
            }
        }
        
        $this->entityManager->flush();
        
        if(count($msg_add) > 0 )
            array_unshift($msg_add, "Feriados replicados:");
        
        if(count($msg_not) > 0)
            if(count($msg_add) > 0)
                array_unshift($msg_not, "<br>Feriados já existentes:");
            else
                array_unshift($msg_not, "Feriados já existentes e não replicados:");
        
        $msg = array_merge($msg_add, $msg_not);
        
        return $msg;
    }
    
    public function getFeriados($nrorg, $nrcalendario, $ano) {
        $sql = "SELECT * FROM OCORCALENDARIO WHERE NRORG = :P_NRORG AND NRCALENDARIO = :P_NRCALENDARIO AND TO_CHAR(DTCALENDARIO, 'YYYY') = :P_ANO";
        $params = array('P_NRORG' => $nrorg, 'P_NRCALENDARIO' => $nrcalendario, 'P_ANO' => $ano);
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        
        return $query->fetchAll();
    }
    
    
}