<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class TipoComiteController extends Crud {
    
    protected $tipoComiteService;
    protected $dataSourceName = 'gpe_tipocomite';
    protected $dataSourceName1 = 'gpe_tipocomitetiporeuniao_get';
    protected $dataSourceName2 = 'gpe_tipocomiteocup_get';
    protected $dataSourceName3 = 'gpe_tipodireitocupcomite_get';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\SST\Services\TipoComiteService $tipoComiteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->tipocomiteService = $tipoComiteService;
        parent::__construct($dataSourceManager);
    }

    public function getTipoComite(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setTipoComite(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTIPOCOMITE', $row)) {
                    if ($row['NRTIPOCOMITE'] === '') {
                        $row['NRTIPOCOMITE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOCOMITE', 1, 12, true);
                    }
                } else {
                    $row['NRTIPOCOMITE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOCOMITE', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getTipoComiteReuniao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setTipoComiteReuniao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_tipocomitetiporeuniao";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTIPOCOMITETIPOREUNIAO', $row)) {
                    if ($row['NRTIPOCOMITETIPOREUNIAO'] === '') {
                        $row['NRTIPOCOMITETIPOREUNIAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOCOMITETIPOREUNIAO', 1, 12, true);
                    }
                } else {
                    $row['NRTIPOCOMITETIPOREUNIAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOCOMITETIPOREUNIAO', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getTipoComiteOcup(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setTipoComiteOcup(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_tipocomiteocup";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTIPOCOMITEOCUP', $row)) {
                    if ($row['NRTIPOCOMITEOCUP'] === '') {
                        $row['NRTIPOCOMITEOCUP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOCOMITEOCUP', 1, 12, true);
                    }
                } else {
                    $row['NRTIPOCOMITEOCUP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOCOMITEOCUP', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getTipoDireitoOcup(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName3);
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setTipoDireitoOcup(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_tipodireitocupcomite";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTIPODIREITOOCUPCOMITE', $row)) {
                    if ($row['NRTIPODIREITOOCUPCOMITE'] === '') {
                        $row['NRTIPODIREITOOCUPCOMITE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPODIREITOOCUPCOMITE', 1, 12, true);
                    }
                } else {
                    $row['NRTIPODIREITOOCUPCOMITE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPODIREITOOCUPCOMITE', 1, 12, true);
                }
            }
            $this->tipocomiteService->validaTipoDireitoCadastrado($row);
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->tipocomiteService->deleteTipoComite($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }

    public function deleteTipoDireitoOcup(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "gpe_tipodireitocupcomite";
        parent::delete($request, $response);
    }
    
    public function deleteTipoComiteReuniao(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "gpe_tipocomitetiporeuniao";
        parent::delete($request, $response);
    }
    
    public function deleteTipoComiteOcup(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->tipocomiteService->deleteOcupacao($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 